--
-- Imagic top-level module.  This handles SDRAM and Nios system routing
--
-- From code produced by Stephen A. Edwards, 
-- Columbia University, sedwards@cs.columbia.edu
--
-- From an original by Terasic Technology, Inc.
-- (DE2_TOP.v, part of the DE2 system board CD supplied by Altera)
--

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity Imagic is

  port (
    -- Clocks
    
    CLOCK_27,                                      -- 27 MHz
    CLOCK_50,                                      -- 50 MHz
    EXT_CLOCK : in std_logic;                      -- External Clock

    -- Buttons and switches
    
    KEY : in std_logic_vector(3 downto 0);         -- Push buttons
    SW : in std_logic_vector(17 downto 0);         -- DPDT switches

    -- LED displays

    HEX0, HEX1, HEX2, HEX3, HEX4, HEX5, HEX6, HEX7 -- 7-segment displays
       : out std_logic_vector(6 downto 0);
    LEDG : out std_logic_vector(8 downto 0);       -- Green LEDs
    LEDR : out std_logic_vector(17 downto 0);      -- Red LEDs

    -- RS-232 interface

    UART_TXD : out std_logic;                      -- UART transmitter   
    UART_RXD : in std_logic;                       -- UART receiver

    -- IRDA interface

--    IRDA_TXD : out std_logic;                      -- IRDA Transmitter
    IRDA_RXD : in std_logic;                       -- IRDA Receiver

    -- SDRAM
   
    DRAM_DQ : inout std_logic_vector(15 downto 0); -- Data Bus
    DRAM_ADDR : out std_logic_vector(11 downto 0); -- Address Bus    
    DRAM_LDQM,                                     -- Low-byte Data Mask 
    DRAM_UDQM,                                     -- High-byte Data Mask
    DRAM_WE_N,                                     -- Write Enable
    DRAM_CAS_N,                                    -- Column Address Strobe
    DRAM_RAS_N,                                    -- Row Address Strobe
    DRAM_CS_N,                                     -- Chip Select
    DRAM_BA_0,                                     -- Bank Address 0
    DRAM_BA_1,                                     -- Bank Address 0
    DRAM_CLK,                                      -- Clock
    DRAM_CKE : out std_logic;                      -- Clock Enable

    -- FLASH
    
    FL_DQ : inout std_logic_vector(7 downto 0);      -- Data bus
    FL_ADDR : out std_logic_vector(21 downto 0);  -- Address bus
    FL_WE_N,                                         -- Write Enable
    FL_RST_N,                                        -- Reset
    FL_OE_N,                                         -- Output Enable
    FL_CE_N : out std_logic;                         -- Chip Enable

    -- SRAM
    
    SRAM_DQ : inout std_logic_vector(15 downto 0); -- Data bus 16 Bits
    SRAM_ADDR : out std_logic_vector(17 downto 0); -- Address bus 18 Bits
    SRAM_UB_N,                                     -- High-byte Data Mask 
    SRAM_LB_N,                                     -- Low-byte Data Mask 
    SRAM_WE_N,                                     -- Write Enable
    SRAM_CE_N,                                     -- Chip Enable
    SRAM_OE_N : out std_logic;                     -- Output Enable

    -- USB controller
    
    OTG_DATA : inout std_logic_vector(15 downto 0); -- Data bus
    OTG_ADDR : out std_logic_vector(1 downto 0);    -- Address
    OTG_CS_N,                                       -- Chip Select
    OTG_RD_N,                                       -- Write
    OTG_WR_N,                                       -- Read
    OTG_RST_N,                                      -- Reset
    OTG_FSPEED,                     -- USB Full Speed, 0 = Enable, Z = Disable
    OTG_LSPEED : out std_logic;     -- USB Low Speed, 0 = Enable, Z = Disable
    OTG_INT0,                                       -- Interrupt 0
    OTG_INT1,                                       -- Interrupt 1
    OTG_DREQ0,                                      -- DMA Request 0
    OTG_DREQ1 : in std_logic;                       -- DMA Request 1   
    OTG_DACK0_N,                                    -- DMA Acknowledge 0
    OTG_DACK1_N : out std_logic;                    -- DMA Acknowledge 1

    -- 16 X 2 LCD Module
    
    LCD_ON,                     -- Power ON/OFF
    LCD_BLON,                   -- Back Light ON/OFF
    LCD_RW,                     -- Read/Write Select, 0 = Write, 1 = Read
    LCD_EN,                     -- Enable
    LCD_RS : out std_logic;     -- Command/Data Select, 0 = Command, 1 = Data
    LCD_DATA : inout std_logic_vector(7 downto 0); -- Data bus 8 bits

    -- SD card interface
    
    SD_DAT,                     -- SD Card Data
    SD_DAT3,                    -- SD Card Data 3
    SD_CMD : inout std_logic;   -- SD Card Command Signal
    SD_CLK : out std_logic;     -- SD Card Clock

    -- USB JTAG link
    
    TDI,                        -- CPLD -> FPGA (data in)
    TCK,                        -- CPLD -> FPGA (clk)
    TCS : in std_logic;         -- CPLD -> FPGA (CS)
    TDO : out std_logic;        -- FPGA -> CPLD (data out)

    -- I2C bus
    
    I2C_SDAT : inout std_logic; -- I2C Data
    I2C_SCLK : out std_logic;   -- I2C Clock

    -- PS/2 port

    PS2_DAT,                    -- Data
    PS2_CLK : in std_logic;     -- Clock

    -- VGA output
    
    VGA_CLK,                                            -- Clock
    VGA_HS,                                             -- H_SYNC
    VGA_VS,                                             -- V_SYNC
    VGA_BLANK,                                          -- BLANK
    VGA_SYNC : out std_logic;                           -- SYNC
    VGA_R,                                              -- Red[9:0]
    VGA_G,                                              -- Green[9:0]
    VGA_B : out std_logic_vector(9 downto 0);           -- Blue[9:0]

    --  Ethernet Interface
    
    ENET_DATA : inout std_logic_vector(15 downto 0);    -- DATA bus 16Bits
    ENET_CMD,           -- Command/Data Select, 0 = Command, 1 = Data
    ENET_CS_N,                                          -- Chip Select
    ENET_WR_N,                                          -- Write
    ENET_RD_N,                                          -- Read
    ENET_RST_N,                                         -- Reset
    ENET_CLK : out std_logic;                           -- Clock 25 MHz
    ENET_INT : in std_logic;                            -- Interrupt
    
    -- Audio CODEC
    
    AUD_ADCLRCK : inout std_logic;                      -- ADC LR Clock
    AUD_ADCDAT : in std_logic;                          -- ADC Data
    AUD_DACLRCK : inout std_logic;                      -- DAC LR Clock
    AUD_DACDAT : out std_logic;                         -- DAC Data
    AUD_BCLK : inout std_logic;                         -- Bit-Stream Clock
    AUD_XCK : out std_logic;                            -- Chip Clock
    
    -- Video Decoder
    
    TD_DATA : in std_logic_vector(7 downto 0);  -- Data bus 8 bits
    TD_HS,                                      -- H_SYNC
    TD_VS : in std_logic;                       -- V_SYNC
    TD_RESET : out std_logic;                   -- Reset
    
    -- General-purpose I/O
    
    GPIO_0,                                      -- GPIO Connection 0
    GPIO_1 : inout std_logic_vector(35 downto 0) -- GPIO Connection 1   
    );
  
end Imagic;

architecture rtl of Imagic is

  component sdram_pll is
	PORT
	(
		signal inclk0  : IN STD_LOGIC;
		signal c0	   : OUT STD_LOGIC 
	);
  end component;

  component nios_system is
   port (
-- ++ WED FINAL PROJECT
   -- Add PORTs to nios_system created by SOPC
   -- Global signals
   signal clk      : IN STD_LOGIC;
   signal reset_n  : IN STD_LOGIC;

   -- the_mmc_clk1
   signal SD_CLK_from_the_mmc_clk1 : OUT STD_LOGIC;

   -- the_mmc_datain1
   signal SD_CMD_from_the_mmc_datain1 : OUT STD_LOGIC;

   -- the_mmc_dataout1
   signal SD_DAT_to_the_mmc_dataout1 : IN STD_LOGIC;

   -- the_mmc_ncs1
   signal SD_DAT3_from_the_mmc_ncs1 : OUT STD_LOGIC;

   -- the_sram
   signal SRAM_ADDR_from_the_vga_raster_0        : OUT STD_LOGIC_VECTOR (17 DOWNTO 0);
   signal SRAM_CE_N_from_the_vga_raster_0        : OUT STD_LOGIC;
   signal SRAM_DQ_to_and_from_the_vga_raster_0  : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
   signal SRAM_LB_N_from_the_vga_raster_0       : OUT STD_LOGIC;
   signal SRAM_OE_N_from_the_vga_raster_0       : OUT STD_LOGIC;
   signal SRAM_UB_N_from_the_vga_raster_0       : OUT STD_LOGIC;
   signal SRAM_WE_N_from_the_vga_raster_0       : OUT STD_LOGIC;

   -- the_sdram
   signal zs_addr_from_the_sdram : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
   signal zs_ba_from_the_sdram : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
   signal zs_cas_n_from_the_sdram : OUT STD_LOGIC;
   signal zs_cke_from_the_sdram : OUT STD_LOGIC;
   signal zs_cs_n_from_the_sdram : OUT STD_LOGIC;
   signal zs_dq_to_and_from_the_sdram : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
   signal zs_dqm_from_the_sdram : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
   signal zs_ras_n_from_the_sdram : OUT STD_LOGIC;
   signal zs_we_n_from_the_sdram : OUT STD_LOGIC;

   -- the_vga_raster_0
   signal VGA_BLANK_from_the_vga_raster_0 : OUT STD_LOGIC;
   signal VGA_B_from_the_vga_raster_0     : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
   signal VGA_CLK_from_the_vga_raster_0   : OUT STD_LOGIC;
   signal VGA_G_from_the_vga_raster_0     : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
   signal VGA_HS_from_the_vga_raster_0    : OUT STD_LOGIC;
   signal VGA_R_from_the_vga_raster_0     : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
   signal VGA_SYNC_from_the_vga_raster_0  : OUT STD_LOGIC;
   signal VGA_VS_from_the_vga_raster_0    : OUT STD_LOGIC
-- -- WED FINAL PROJECT
  );
  end component;

  signal clk25 : std_logic := '0';

begin

  nios: nios_system port map (
-- ++ WED FINAL PROJECT
-- Route data to nios_system (for VGA Periphial + SDRAM)
    reset_n => '1',
    clk => CLOCK_50,

	SD_CLK_from_the_mmc_clk1 => SD_CLK,
    SD_CMD_from_the_mmc_datain1 => SD_CMD,
    SD_DAT_to_the_mmc_dataout1 => SD_DAT,
	SD_DAT3_from_the_mmc_ncs1 => SD_DAT3,

    VGA_CLK_from_the_vga_raster_0     => VGA_CLK,
    VGA_HS_from_the_vga_raster_0      => VGA_HS,
    VGA_VS_from_the_vga_raster_0      => VGA_VS,
    VGA_BLANK_from_the_vga_raster_0   => VGA_BLANK,
    VGA_SYNC_from_the_vga_raster_0    => VGA_SYNC,
    VGA_R_from_the_vga_raster_0       => VGA_R,
    VGA_G_from_the_vga_raster_0       => VGA_G,
    VGA_B_from_the_vga_raster_0       => VGA_B,
    SRAM_ADDR_from_the_vga_raster_0   => SRAM_ADDR,
    SRAM_CE_N_from_the_vga_raster_0   => SRAM_CE_N,
    SRAM_DQ_to_and_from_the_vga_raster_0   => SRAM_DQ,
    SRAM_LB_N_from_the_vga_raster_0   => SRAM_LB_N,
    SRAM_OE_N_from_the_vga_raster_0   => SRAM_OE_N,
    SRAM_UB_N_from_the_vga_raster_0   => SRAM_UB_N,
    SRAM_WE_N_from_the_vga_raster_0   => SRAM_WE_N,
 
    -- the_sdram
    zs_addr_from_the_sdram => DRAM_ADDR, 
    zs_ba_from_the_sdram(1) => DRAM_BA_1,
    zs_ba_from_the_sdram(0) => DRAM_BA_0,  
    zs_cas_n_from_the_sdram => DRAM_CAS_N,
    zs_cke_from_the_sdram => DRAM_CKE,
    zs_cs_n_from_the_sdram => DRAM_CS_N,
    zs_dq_to_and_from_the_sdram => DRAM_DQ,
    zs_dqm_from_the_sdram(1) => DRAM_UDQM,
    zs_dqm_from_the_sdram(0) => DRAM_LDQM,
    zs_ras_n_from_the_sdram => DRAM_RAS_N,
    zs_we_n_from_the_sdram => DRAM_WE_N
-- -- WED LAB3
  );

  pll_clock: sdram_pll port map (inclk0 => CLOCK_50, 
                                 c0 => DRAM_CLK);

  HEX7     <= "0001001";                -- Leftmost
  HEX6     <= "0000110";
  HEX5     <= "1000111";
  HEX4     <= "1000111";
  HEX3     <= "1000000";
  HEX2     <= (others => '1');
  HEX1     <= (others => '1');
  HEX0     <= (others => '1');          -- Rightmost
  LEDG     <= (others => '1');
  LEDR     <= (others => '1');
  LCD_ON   <= '1';
  LCD_BLON <= '1';

  -- Set all bidirectional ports to tri-state
  FL_DQ       <= (others => 'Z');
 -- SRAM_DQ     <= (others => 'Z');  -- ++ WED FINAL PROJECT -- Controlled in periphial
  OTG_DATA    <= (others => 'Z');
  LCD_DATA    <= (others => 'Z');
  SD_DAT      <= 'Z';
  I2C_SDAT    <= 'Z';
  ENET_DATA   <= (others => 'Z');
  AUD_ADCLRCK <= 'Z';
  AUD_DACLRCK <= 'Z';
  AUD_BCLK    <= 'Z';
  GPIO_0      <= (others => 'Z');
  GPIO_1      <= (others => 'Z');

end rtl;
